## What is Weaver?

<!-- ![Weaver deployment](src/img/what_is_weaver.png) -->

Weaver provides blueprint-driven specification and deployment of IT environments by specifying "Environments as Code". 
An environment is characterized by a set of computing resources including software, virtual or physical platforms, and 
network configuration. In an IT organization you typically have many environments. Weaver provides a context for management
and organization of those environments such as QA, pre-production, and production. 

Technically, Weaver is a layer on top of configuration management technologies such as {http://docs.opscode.com/ Chef} or 
{http://puppetlabs.com Puppet} that embrace the paradigm of "infrastructure as code". While these technologies 
are well-suited for describing how to configure individual machines in an environment, they do not provide a means
for defining the context of how different machines relate to each other, what automation needs to run on each machine
and how parameters need to be passed between the automation that runs on each machine. However, this is crucial to 
understand how to construct an environment and a reliable and repeatable manner.  

Weaver is implemented as a {http://en.wikipedia.org/wiki/Domain-specific_language Domain Specific Language (DSL)} in Ruby 
and can be effectively used in combination with different forms of automation (e.g., Chef Recipes or Bash scripts). 
Currently, Weaver supports deployment to IBM Workload Deployer (Virtual System Patterns), and several IaaS clouds 
(OpenStack and Amazon).

## Getting Started

To get started with Weaver, we **recommend** you to look at the set of {file:doc/src/Tutorial.md tutorials}.   

After that check out links below provide more information to learn about the installation, the language, CLI,  runtime. The 
purpose of this documentation is for internal users. The capabilities reflected here are partially experimental
and not ready for product use. If you looking for a Weaver version that is part of an IBM product, 
see [Product Usage](#ProductUsage) below.

* {file:doc/src/Tutorial.md Tutorial} 
* {file:doc/src/Installation.md Installation}
* {file:doc/src/Language.md Language}
* {file:doc/src/CLI.md CLI Reference}
* {file:doc/src/Testing.md Testing and Problem Determination}
* {file:doc/src/Runtime.md Runtime and REST API}

Internal information for the development team:

* {file:doc/src/Development.md Development}
* {file:doc/src/Internals.md Internals}

## Report a Problem

If you have any problem with Weaver, please report it in the {https://w3-connections.ibm.com/communities/service/html/community/bookmarks?communityUuid=87ddcbf4-e3ae-477f-8de7-4f2d3efbaa79&successMessage=label.addto.confirm#fullpageWidgetId=W9198fd8d856a_4fc3_bf0f_dfdeee874216&forumsPg=null&topicsPg=null  Weaver Forum on W3 
Connections}. Please describe the problem as concise as possible so we can help 
effectively. Don't be shy, ask anything you want, even if you need better or more
documentation. We appreciate all sort of feedback, good or bad ;).


## <a id='ProductUsage'/>Product Usage

Weaver is part of the {https://jazz.net/products/smartcloud-continuous-delivery/ IBM SmartCloud Continuous Delivery (SCD)} product. For
detailed information on its use in SCD, please look at the {http://pic.dhe.ibm.com/infocenter/sccdhelp/v1r0/index.jsp official product documentation}.
The SCD capabilities are a great enhancement in the {https://www.ibm.com/developerworks/mydeveloperworks/blogs/devops/entry/welcome_to_the_enterprise_devops_blog3?lang=en Enterprise DevOps} 
space. However, Weaver can be used without the SCD capabilities as well by using its CLI to validate and deploy environments.   

For more information see the other pages as mentioned above or see the following links. 
The {https://w3-connections.ibm.com/communities/community/weaver Weaver Community} has a forum to ask questions and get in contact with developers. 
The official {https://jazz.net/products/smartcloud-continuous-delivery Jazz Community} has more up-to-date 
information about Weaver and ongoing development efforts related to its use in IBM SmartCloud Continuous Delivery (SCD).

Please keep in mind that the capabilities of the Weaver language and runtime as documented on these pages <b>may go beyond the available capabilities of 
Weaver in SCD</b>. We have experimental capabilities that are not in the product, such as support for OpenStack and Amazon. 
The documentation will state that accordingly to avoid any confusion.  

## Updating and building this documentation

If you find any bugs in this documentation, please report or, if you are a developer on the project, update and rebuild the documentation using the following
command: 

    !!!bash
	rake weaver:yard # builds the documentation from scratch
	rake weaver:deploy_doc # push the newly built documentation to the documentation server 

If you are doing interactive editing locally do the following: 
	
    !!!bash
	yard server -r # -r allows reload to revise the doc automatically if you make changes

Browse to [http://localhost:8808/](http://localhost:8808/).  Reload browser if you make changes to the documentation.
  

## Copyright

Copyright (c) 2013 IBM Corporation

