require 'json'

require 'weaver/chef/chef_node_inspector'
require 'weaver/core/logging'
require 'weaver/dsl/weaver_parser'

module Weaver
  module Chef
    #
    # Introspects \Chef role files comparing them against {Automation}
    # generated by {ChefNodeInspector} for use by the +:check_automation_changed+ builtin {Rule}.
    class RoleComparator

      # @param [Automation] automation_model Automation instance
      # @param [String] chef_rolefile Chef *_role.rb file
      # @param [Rule] reporter a Rule to call report() on when problems are found
      def self.role_compare(automation_model, chef_rolefile, reporter)
        
        # Generate a new model from the rolefile
        chef_dir = File.dirname(File.dirname(chef_rolefile))
        # puts "RoleComparator.role_compare, chef_dir = #{chef_dir}"
        inspector = Weaver::Chef::ChefNodeInspector.new([chef_dir]) { |level, msg| }
        begin
          recreated_root = inspector.model_role(File.basename(chef_rolefile), '.')
        rescue Weaver::Core::WeaverError => e
          reporter.report(Logger::INFO, "Did not find role file #{chef_rolefile}; #{e.to_s}", automation_model)
          return
        end
        recreated_model = recreated_root.automation.values[0] 
 
        unmodeled_attributes = Set.new(recreated_model.amd.keys).subtract(automation_model.amd.keys)
        unmodeled_attributes.each { |id| reporter.report(Logger::INFO, "Attribute #{id.inspect} appears in the Chef recipe #{chef_rolefile} but not in the Weaver model", automation_model) }
        
        nonfunctional_attributes = Set.new(automation_model.amd.keys).subtract(recreated_model.amd.keys)
        nonfunctional_attributes.each { |id| reporter.report(Logger::WARN, "Attribute #{id.inspect} is no longer present in the Chef recipe #{chef_rolefile}", automation_model) }
      end
      
    end
  end      
end 