require 'rubygems'
require 'thor'
require 'thor/group'
require 'set'
require 'weaver/utils/plugin_loader'
require 'weaver/cli/weaver_cli_plugin'

module Weaver
  # Weaver's \Cli module implements a command line interface for working with Weaver files.
  #
  # The base command is <em>weaver \<domain\> \<command\></em> where _domain_ is +env+ (for Environment),
  # +iwd+ (for IBM Workload Deployer), +key+ (for cryptographic key files), +chef+ (for Chef).
  module Cli
    # Main command line tool for Weaver
    #
    # Syntax: weaver <action> [options]
    #
    # Commands are corresponding to the methods below. Issuing the weaver command
    # without parameters will print a list of available commands.
    #
    # Author:: Florian Rosenberg
    #
    class WeaverCli < Thor

      desc "version", "Prints Weaver version"
      def version
        puts "Weaver version #{File.new(File.expand_path(File.dirname(__FILE__) + '/../../../VERSION')).read()}"
      end
      
      # Load all plugins as Class objects
      loader = Weaver::Utils::PluginLoader.new(
        [File.join("weaver","cli","plugins"), File.join("weaver","plugins","cli")],
        [File.expand_path(File.join("..","plugins",""), __FILE__)]
      )
      loader.load_plugins

      # Iterate through all plugins and register them.
      Weaver::Cli::WeaverCliPlugin.plugins.each do |p|
        #puts "args: #{p.namespace}"
        register p, p.namespace , p.namespace, p.get_description
      end
    end
  end
end