require 'weaver/core/base.rb'

module Weaver
  module Core
    # Holds firewall configuration for a Node.  Created from the Weaver +firewall+ statement.
    # 
    # @see Weaver::Dsl::FirewallStatement 
    class Firewall
      include Weaver::Core::Generation

      # FirewallRule array for TCP communication 
      attr_accessor :tcp
      
      # FirewallRule array for UDP communication 
      attr_accessor :udp
      
      # Create empty (rule-less) Firewall model object
      def initialize()
        @tcp = []
        @udp = []
      end

      # Describe firewall
      def to_s
        "firewall tcp=#{@tcp} udp=#{@udp}"
      end
      
      # Validates the entity and its direct and indirect contained children.
      # * +out_messages+ ProblemStatus will be pushed onto this for any validation message
      # * +desired_phases+ optional Array which may be +[:predefined]:
      def validate(out_messages, desired_phases = nil)
#        @@logger.debug "    TODO: validate Firewall"
      end
      
    end
    
    # Holds netmask and port configuration for a Firewall.  Created from the Weaver +firewall+ statement.
    # 
    # @see Weaver::Dsl::FirewallStatement 
    class FirewallRule
      
      # Array of port Fixnum or Range
      attr_accessor :ports
      
      # Net mask String such as '0.0.0.0/0'
      attr_accessor :netmask
      
      # <code>:ingress</code> or <code>:egress</code>
      attr_accessor :direction
      
      # Create empty FirewallRule
      # * +ports+ Array of port Fixnum or Range
      # * +netmask+ optional Net mask String such as '0.0.0.0/0'
      # * +direction+ +nil+, <code>:ingress</code> or <code>:egress</code>
      def initialize(ports, netmask=nil, direction=nil)
        @ports = ports
        @netmask = netmask
        @direction = direction
      end
      
      # Returns <code>true</code> if +another_base+ is equal by value.
      def ==(another_rule)
        if !another_rule.kind_of?(FirewallRule)
          return false
        end
        
        return @ports == another_rule.ports && @netmask == another_rule.netmask && 
          @direction == another_rule.direction
      end
    end
  end
end