require 'weaver/core/base.rb'

module Weaver
  module Core
    # Represents a link type such as config_link
    # @see LinkStatement
    #
    # Author:: Ed Snible
    # Author:: Florian Rosenberg
    # Copyright:: IBM
    class Link < Base
      include Weaver::Core::Generation

      # Source of the link
      attr_accessor :source, :target
      
      attr_accessor :firewall
      
      # * +id+ Symbol to represent the entity
      # * +parent+ The enclosing Base
      def initialize(id, parent)
        super(id, parent)
        @type = :link
      end

      # Validates the entity and its direct and indirect contained children.
      # * +out_messages+ ProblemStatus will be pushed onto this for any validation message
      # * +desired_phases+ optional Array which may be +[:predefined]:
      def validate_collect(out_messages, desired_phases = nil)
        @@logger.debug "Validating link '#{id}'"
        @@logger.debug "  has name '#{name}'"
        
        super

        unless source.kind_of?(Base)
          out_messages.push Weaver::Validation::ProblemStatus.new(Logger::ERROR, "Unknown link source '#{source}'", self)
        end

        unless target.kind_of?(Base)
          out_messages.push Weaver::Validation::ProblemStatus.new(Logger::ERROR, "Unknown link target '#{target}'", self)
        end

        if source.equal?(target)
          out_messages.push Weaver::Validation::ProblemStatus.new(Logger::ERROR, "link source and target are the same", self)
        end
      end

    end
  end
end