require 'weaver/dsl/abstract_statement'
require 'weaver/dsl/requires/type_handler_registry'
require 'weaver/dsl/automation_container'
require 'weaver/dsl/component_container'
require 'weaver/dsl/load_driver_container'
require 'weaver/dsl/policy_container'
require 'weaver/dsl/service_container_statement'
require 'weaver/dsl/provider_container_statement'


module Weaver
  module Dsl
    # The ComponentStatement implements the processing of the language element 'component'.
    # Nodes support statements: requires
    #
    # Author:: Ed Snible
    # Author:: Florian Rosenberg
    # @see Component
    class ComponentStatement < AbstractStatement
      include LoadDriverContainer
      include ServiceContainerStatement
      include PolicyContainer
      include AutomationContainer
      include ComponentContainer
      include ProviderContainerStatement
    
      # * +id+ Symbol representing the entity, or a Core::Component which is the entity
      # * +parent_statement+ The enclosing AbstractStatement
      def initialize(id, parent_statement)
        @@logger.debug "component[id: #{id} on #{parent_statement.model.type} :#{parent_statement.model.id}]"
        super(id, :component, parent_statement)
      end

      # Language +requires+ statement
      # +requires_data+ a Hash
      def requires(requires_data)
        @@logger.debug "requires [#{requires_data}]"               
        handler = Weaver::Dsl::Requires::TypeHandlerRegistry.instance.find(requires_data.keys[0])        
        unless handler 
          return model.add_validation_failure("Cannot find an appropriate 'requires' handler.")
        end
        
        # expect to have one handler
        requirement = handler.handle(self, requires_data)
        requirement.weaver_origin = weaver_stackframe
        
        return model.add_validation_failure("Can't create requirement from #{requires_data}") unless requirement.kind_of?(Requirement)

        @model.requirements.push(requirement)
      end

    end
  end
end