require 'weaver/core/logging'

module Weaver
  module Dsl
    # Mix in GroupContainer in statements that may contain {GroupStatement}
    #
    # Author:: Ed Snible
    # Copyright:: IBM
    module GroupContainer
      include Weaver::Core
      
      # * *Args* :
      #   -+base+ the AbstractStatement being mixed with
      def self.included(base)
        base.register_statement_handler([:group_handler])
      end

      # * *Args* :
      #   -+sym+ Symbol representing Weaver type
      #   -+id+ Symbol naming entity
      #   -+block+ optional block of additional metadata
      def group_handler(sym, id, &block)
        unless is_a?(Weaver::Dsl::GroupContainer) && root_statement.model.weaver_extended_type?(sym, Group)
          return nil
        end
        group(id, root_statement.model.prototype_instance(sym, id, model, weaver_stackframe), &block)
      end

      # If prototype is nil, we will create a core Group, otherwise inherit attributes,
      # default attribute values, and validators from the prototype
      def group(id, prototype = nil, &block)
        @@logger.debug "group[(#{prototype.id.to_s+') ' unless prototype.nil?}:#{id}]"
        return if fail_id_redefinition(id, Group)
        group = GroupStatement.new(id, self)
        unless prototype.nil?
          group.model = prototype
          group.model.id = id
          group.model.scope = @model
        end
        @model.groups[id] = group.model
        group.instance_eval &block unless block.nil?
        return group.model
      end
    end

  end
end
