module Weaver
  module Dsl
    # Mix in LinkContainer in statements that may contain {LinkStatement}
    #
    # Author:: Ed Snible
    # Author:: Florian Rosenberg
    # Copyright:: IBM
    module LinkContainer
      include Weaver::Core
      
      # * *Args* :
      #   -+base+ the AbstractStatement being mixed with
      def self.included(base)
        base.register_statement_handler([:link_handler])
      end

      # * *Args* :
      #   -+sym_or_model+ <Symbol representing Weaver type, e.g. <code>:config_link</code> | Core::Link
      #   -+id+ Symbol naming entity
      #   -+block+ optional block of additional metadata
      def link_handler(sym_or_model, id, &block)
        return nil unless is_a?(LinkContainer) && root_statement.model.weaver_extended_type?(sym_or_model, Link)
        return nil unless id.kind_of?(Symbol) 
        
        prototype = sym_or_model.kind_of?(Symbol) ? root_statement.model.prototype_instance(sym_or_model, id, model, weaver_stackframe) : sym_or_model 
        link(id, prototype, &block)
      end

      # If prototype is nil, we will create a core Link, otherwise inherit attributes,
      # default attribute values, and validators from the prototype
      def link(id, prototype = nil, &block)
        @@logger.debug "link[(#{prototype.id.to_s+') ' unless prototype.nil?}:#{id}]"
        return if fail_id_redefinition(id, Link)
        link_model = @model.links[id]
        if link_model.nil?
          link_model = prototype || root_statement.model.prototype_instance(:link, id, model, weaver_stackframe)
          @model.links[id] = link_model
        else
          root_statement.model.widen(link_model, prototype) unless prototype.nil?
        end
        
        unless block.nil?
          link = LinkStatement.new(link_model, self)
          link.instance_eval &block
        end

        return link.model
      end
    end
  end
end