require 'weaver/core/service'
require 'weaver/dsl/abstract_statement'

module Weaver
  module Dsl
    
    # The ServiceStatement implements the processing of the language element 'service'.
    # Nodes support statements: +predeploy+, +deploy+, +postdeploy+, +undeploy+
    # @see Weaver::Core::Service
    #
    # Author:: Ed Snible
    # Copyright:: IBM
    class ServiceStatement < AbstractStatement
      
      # * +id+ Symbol representing the entity, or a Core::Link which is the entity
      # * +parent_statement+ The enclosing AbstractStatement
      def initialize(id, parent_statement)
        @@logger.debug "service[id: #{id}]"
        super(id, Weaver::Core::Service, parent_statement)
      end

      # Summarize model
      def to_s
        "service[id: #{@id}, name: #{@name}, description: #{@description}]"
      end

      # Parse +deploy+ language statement
      # * +dataflow+ Attributes affected by the deployment
      # * +block+ ServiceStatement instance definition 
      def deploy(dataflow={}, &block)
        @@logger.debug "deploy[#{dataflow.inspect}]"

        model.deploy_dataflow = dataflow
        model.deploy_block = block        
      end
      
      # Parse +predeploy+ language statement
      # * +dataflow+ Attributes affected by the deployment
      # * +block+ ServiceStatement instance definition 
      def predeploy(dataflow={}, &block)
        @@logger.debug "predeploy[#{dataflow.inspect}]"

        model.predeploy_dataflow = dataflow
        model.predeploy_block = block        
      end
      
      # Parse +postdeploy+ language statement
      # * +dataflow+ Attributes affected by the deployment
      # * +block+ ServiceStatement instance definition 
      def postdeploy(dataflow={}, &block)
        @@logger.debug "postdeploy[#{dataflow.inspect}]"

        model.postdeploy_dataflow = dataflow
        model.postdeploy_block = block        
      end
      
      # Parse +undeploy+ language statement
      # * +dataflow+ Attributes affected by the deployment
      # * +block+ ServiceStatement instance definition 
      def undeploy(dataflow={}, &block)
        @@logger.debug "deploy[#{dataflow.inspect}]"

        model.undeploy_dataflow = dataflow
        model.undeploy_block = block        
      end

    end
  end
end