
require 'string'

module Weaver
  module Utils
    # This class has helper methods for Weaver packages
    class PackageHandler
      include Weaver::Core

      # Instantiates a package handler
      # * *Args*    :
      # - +node+ -> the node that the PackageHandler will handle
      def initialize(node)
        @node = node
      end


      # Returns a list of the packages that are included in the node,
      # including packages at the node level and at the component level
      # * *Returns* :
      #   - an array of packages
      def get_packages()
        packages = []
        unless @node.nil?
          packages = @node.flatten_children.select { |ent| ent.type == :package } 
        end
        return packages
      end
      
      # Returns a json Array that has a list of hashes, each representing a package.
      # Each package hash includes the attributes for downloading the package, including
      # provisioner and package attributes.
      # For example, 
      # * *Returns* :
      #   - an Json array with a Hash for each package that specifies download attributes
      def generate_package_json()
        json_array = []
        get_packages.each do |package|
          provisioner = package.effective_provisioner(:artifact)
          if provisioner.nil?
            @logger.warning "Package '#{package.id}' does not have a provider. Modify the package in the weaver file by specifying the provider with the statement 'provider: <provider_id>"
          else
            exclude_attributes = [:name, :description, :provider_class]
            hash = {:type => "#{provisioner.provider_type}"}
            provisioner.amd.each do | id, attr|
              unless exclude_attributes.include?(id)
              # unless attr.implicit?
                hash[id] = attr.value if !attr.value.nil?
              end
            end
            package.amd.each do | id, attr|
              unless exclude_attributes.include?(id)
              # unless attr.implicit?
                hash[id] = attr.value if !attr.value.nil?
              end
            end
            json_array << hash
          end
        end
        return json_array
      end


    end
  end

end
